@@ -1,7 +1,7 @@
 import asyncio
 from types import FunctionType
 import websockets
-import json
+import orjson
 import logging
 from .lib.message import WsMessage
 from .lib.payload import Payloads, MessagePayload
@@ -15,7 +15,6 @@ from typing import (
     TypeVar,
     Union,
     Dict,
-    Tuple,
 )
 
 logger = logging.getLogger(__name__)
@@ -32,19 +31,26 @@ class Client:
         This should be unique to all the clients.
     port: Optional[:class:`int`]
         The port on which the server is running. Defaults to 13254.
+    reconnect: Optional[:class:`bool`]
+        If set to True, the client will automatically try to reconnect to the winerp server
+        every 60 seconds (default). This option is set to True by default.
     """
     def __init__(
         self,
         local_name: str,
-        port: int = 13254
+        port: int = 13254,
+        reconnect: bool = True
     ):
-        self.uri = f"ws://localhost:{port}"        
-        self.local_name = local_name
+        self.uri: str = f"ws://localhost:{port}"        
+        self.local_name: str = local_name
+        self.reconnect: bool = reconnect
+        self.reconnect_threshold: int = 60
+        self.max_data_size: float = 2 #MiB
         self.websocket = None
         self.__routes = {}
         self.listeners = {}
-        self.event_listeners: Dict[str, Tuple[asyncio.Future, Callable]] = {}
-        self._authorized = False
+        self.event_listeners: Dict[str, asyncio.Future] = {}
+        self._authorized: bool = False
         self._on_hold = False
         self.events = [
             "on_winerp_connect",
@@ -56,12 +62,6 @@ class Client:
             "on_winerp_error"
         ]
     
-    async def __empty_event(self, *args, **kwargs):
-        ...
-
-    def get_event(self, key):
-        return self.events.get(key) or self.__empty_event
-    
     @property
     def authorized(self) -> bool:
         '''
@@ -80,11 +80,40 @@ class Client:
         if not isinstance(data, WsMessage):
             data = data.__dict__
 
-        await self.websocket.send(json.dumps(data))
+        await self.websocket.send(orjson.dumps(data).decode("utf-8"))
     
     def __send_message(self, data):
         asyncio.create_task(self.send_message(data))
     
+    async def __verify_client(self):
+        payload = MessagePayload(
+            type = Payloads.verification,
+            id = self.local_name,
+            uuid = str(uuid.uuid4())
+        )
+        await self.send_message(payload)
+        logger.info("Verification request sent")
+
+    async def __connect(self) -> None:
+        if self.websocket is None or self.websocket.closed:
+            logger.info("Connecting to Websocket")
+            self.websocket = await websockets.connect(
+                self.uri, close_timeout=0, ping_interval=None, max_size=int(self.max_data_size*1048576)
+            )
+            self._authorized = False
+            self._dispatch_event('winerp_connect')
+            logger.info("Connected to Websocket")
+
+    async def __reconnect_client(self) -> bool:
+        while True:
+            try:
+                await self.__connect()
+                await self.__verify_client()
+                return True
+            except:
+                logger.debug(f"Failed to reconnect. Retrying in {self.reconnect_threshold}s")
+                await asyncio.sleep(self.reconnect_threshold)
+        
     async def start(self) -> None:
         '''|coro|
 
@@ -100,19 +129,9 @@ class Client:
             :class:`None`
         '''
         if self.websocket is None or self.websocket.closed:
-            logger.info("Connecting to Websocket")
-            self.websocket = await websockets.connect(self.uri, close_timeout=0, ping_interval=None)
-            self._dispatch_event('winerp_connect')
-            logger.info("Connected to Websocket")
-            payload = MessagePayload(
-                type = Payloads.verification,
-                id = self.local_name,
-                uuid = str(uuid.uuid4())
-            )
-            await self.send_message(payload)
-            logger.info("Verification request sent")
+            await self.__connect()
+            await self.__verify_client()
             asyncio.create_task(self.__on_message())
-            logger.info("Listening to messages")
         else:
             raise ConnectionError("Websocket is already connected!")
 
@@ -185,7 +204,7 @@ class Client:
         timeout: int = 60
     ):
         future = loop.create_future()
-        self.listeners[_uuid] = (None, future)
+        self.listeners[_uuid] = future
         return asyncio.wait_for(future, timeout, loop=loop)
 
     async def request(
@@ -325,7 +344,6 @@ class Client:
         self,
         event: str,
         timeout: int = 60,
-        check: Callable = None,
     ):
         '''|coro|
 
@@ -345,10 +363,7 @@ class Client:
             The event to wait for.
         timeout: Optional[:class:`int`]
             Time to wait before raising :class:`~asyncio.TimeoutError`. Defaults to 60.
-        check: Optional[:class:`Callable`]
-            A function to check if the event meets the requirements.
-            If it returns True, the event is returned.
-        
+
         Raises
         -------
             asyncio.TimeoutError
@@ -360,12 +375,6 @@ class Client:
                 The payload for the event that meets the requirements.
         '''
         future = asyncio.get_event_loop().create_future()
-        if check is None:
-
-            def _check(*args):
-                return True
-
-            check = _check
 
         ev = event.lower()
         try:
@@ -374,17 +383,22 @@ class Client:
             listeners = []
             self.event_listeners[ev] = listeners
 
-        listeners.append((future, check))
+        listeners.append(future)
         return asyncio.wait_for(future, timeout)
 
 
     async def __on_message(self):
+        logger.info("Listening to messages")
         while True:
             try:
-                message = WsMessage(json.loads(await self.websocket.recv()))
+                message = WsMessage(orjson.loads(await self.websocket.recv()))
             except websockets.exceptions.ConnectionClosedError:
                 self._dispatch_event('winerp_disconnect')
-                break
+                if self.reconnect:
+                    if not await self.__reconnect_client():
+                        break
+                else:
+                    break
 
             if message.type.success and not self._authorized:
                 logger.info("Authorized Successfully")
@@ -443,11 +457,10 @@ class Client:
         payload = MessagePayload().from_message(message)
         payload.type = Payloads.response
         payload.id = self.local_name
-        payload.data = {}
+
 
         try:
             payload.data = await func(**data)
-            json.dumps(payload.data) #Ensuring data is serializable
         except Exception as error:
             logger.exception(error)
             self._dispatch_event('winerp_error', error)
@@ -460,41 +473,37 @@ class Client:
             payload.data = str(error)
             payload.traceback = traceback_text
         finally:
-            self.__send_message(payload)
+            try:
+                await self.send_message(payload)
+            except TypeError as error:
+                logger.exception("Failed to convert data to json")
+                self._dispatch_event('winerp_error', error)
+                payload.type = Payloads.error
+                payload.data = str(error)
+                payload.traceback = ''.join(
+                    traceback.format_exception(
+                        TypeError,
+                        error,
+                        error.__traceback__
+                    )
+                )
+                self.__send_message(payload)
     
     async def _dispatch(self, msg: WsMessage):
         data = msg.data
-        logger.debug('Dispatch -> %r', data)
         _uuid = msg.uuid
         if _uuid is None:
             raise MissingUUIDError('UUID is missing.')
-        
-        found = False
-        for key, val in self.listeners.items():
-            if key == _uuid:
-                found = True
-                check = val[0]
-                future: asyncio.Future = val[1]
-
-                def _check(*args):
-                    return True
-
-                if check is None:
-                    check = _check
-                
-                if not msg.type.error:
-                    if check(data):
-                        future.set_result(data)
-                    else:
-                        future.set_exception(
-                            CheckFailureError(f"Check failed for UUID {_uuid}")
-                        )
-                else:
-                    future.set_exception(
-                        ClientRuntimeError(msg.data)
-                    )
-        if not found:
+        if _uuid not in self.listeners:
             raise UUIDNotFoundError(f"UUID {_uuid} not found in listeners.")
+        
+        future: asyncio.Future = self.listeners[_uuid]
+        if not msg.type.error:
+            future.set_result(data)
+        else:
+            future.set_exception(
+                ClientRuntimeError(msg.data)
+            )
 
     def event(self, func: Coro, /) -> Coro:
         '''
@@ -528,13 +537,12 @@ class Client:
 
     def _dispatch_event(self, event_name: str, *args, **kwargs):
         logger.debug('Event Dispatch -> %r', event_name)
-        
-        for ev, data in self.event_listeners.items():
-            if ev == event_name:
-                for fut, check in data:            
-                    if check(*args, **kwargs):
-                        fut.set_result(None)
-                        logger.debug('Event %r has been dispatched', event_name)
+        try:
+            for future in self.event_listeners[event_name]:
+                future.set_result(None)
+                logger.debug('Event %r has been dispatched', event_name)
+        except KeyError:
+            ...
 
         try:
             coro = getattr(self, f'on_{event_name}')
